"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.usePrefersColorScheme = exports.default = exports.ColorScheme = void 0;

var _react = require("react");

let ColorScheme;
exports.ColorScheme = ColorScheme;

(function (ColorScheme) {
  ColorScheme["none"] = "";
  ColorScheme["dark"] = "dark";
  ColorScheme["light"] = "light";
})(ColorScheme || (exports.ColorScheme = ColorScheme = {}));

const usePrefersColorScheme = () => {
  const [colorScheme, setColorScheme] = (0, _react.useState)(ColorScheme.none);
  (0, _react.useEffect)(() => {
    if (window.matchMedia('(prefers-color-scheme: dark)').matches) {
      // dark mode
      setColorScheme(ColorScheme.dark);

      const listener = e => {
        const newColorScheme = e.matches ? ColorScheme.dark : ColorScheme.light;
        setColorScheme(newColorScheme);
      };

      window.matchMedia('(prefers-color-scheme: dark)').addEventListener('change', listener);
      return () => {
        window.matchMedia('(prefers-color-scheme: dark)').removeEventListener('change', listener);
      };
    } else if (window.matchMedia('(prefers-color-scheme: light)').matches) {
      // light mode
      setColorScheme(ColorScheme.light);

      const listener = e => {
        const newColorScheme = e.matches ? ColorScheme.light : ColorScheme.dark;
        setColorScheme(newColorScheme);
      };

      window.matchMedia('(prefers-color-scheme: light)').addEventListener('change', listener);
      return () => {
        window.matchMedia('(prefers-color-scheme: light)').removeEventListener('change', listener);
      };
    }
  }, []);
  return colorScheme;
};

exports.usePrefersColorScheme = usePrefersColorScheme;
var _default = usePrefersColorScheme;
exports.default = _default;